searchNodes=[{"ref":"Arangox.html","title":"Arangox","type":"module","doc":"An implementation of :db_connection for ArangoDB. Supports VelocyStream, Active Failover, transactions and cursors. Documentation Tested on: ArangoDB 3.3.9 - 3.5 Elixir 1.6 - 1.9 OTP 20 - 22 Examples iex&gt; {:ok, conn} = Arangox.start_link(pool_size: 10) iex&gt; Arangox.request(conn, :get, &quot;/_admin/server/availability&quot;) {:ok, %Arangox.Request{ body: &quot;&quot;, headers: %{}, method: :get, path: &quot;/_admin/server/availability&quot; }, %Arangox.Response{ body: %{&quot;code&quot; =&gt; 200, &quot;error&quot; =&gt; false, &quot;mode&quot; =&gt; &quot;default&quot;}, headers: %{}, status: 200 }} iex&gt; Arangox.get!(conn, &quot;/_admin/server/availability&quot;) %Arangox.Response{ body: %{&quot;code&quot; =&gt; 200, &quot;error&quot; =&gt; false, &quot;mode&quot; =&gt; &quot;default&quot;}, headers: %{}, status: 200 } iex&gt; Arangox.get(conn, &quot;/invalid&quot;) {:error, %Arangox.Error{ endpoint: &quot;http://localhost:8529&quot;, error_num: 404, message: &quot;unknown path &#39;/invalid&#39;&quot;, status: 404 }} iex&gt; Arangox.transaction(conn, fn c -&gt; iex&gt; stream = iex&gt; Arangox.cursor( iex&gt; c, iex&gt; &quot;FOR i IN [1, 2, 3] FILTER i == 1 || i == @num RETURN i&quot;, iex&gt; %{num: 2}, iex&gt; properties: [batchSize: 1] iex&gt; ) iex&gt; iex&gt; Enum.reduce(stream, [], fn resp, acc -&gt; iex&gt; acc ++ resp.body[&quot;result&quot;] iex&gt; end) iex&gt; end) {:ok, [1, 2]} Peer Dependencies By default, Arangox communicates with ArangoDB via the VelocyStream protocol, which requires the :velocy library: def deps do [ ... {:arangox, &quot;~&gt; 0.1.0&quot;}, {:velocy, &quot;~&gt; 1.1&quot;} ] end The default vst chunk size is 30_720. To change it, you can include the following in your config/config.exs: config :arangox, :vst_maxsize, 12_345 HTTP Arangox has two HTTP clients, Arangox.GunClient and Arangox.MintClient, they require a json library: def deps do [ ... {:arangox, &quot;~&gt; 0.1.0&quot;}, {:jason, &quot;~&gt; 1.1&quot;}, {:gun, &quot;~&gt; 1.3&quot;} # or {:mint, &quot;~&gt; 0.4.0&quot;} ] end Arangox.start_link(client: Arangox.GunClient) # or Arangox.MintClient NOTE: :mint doesn&#39;t support unix sockets. NOTE: Since :gun is an Erlang library, you might need to add it as an extra application in mix.exs: def application() do [ extra_applications: [:logger, :gun]) ] end To use something else, you&#39;d have to implement the Arangox.Client behaviour in a module somewhere and set that instead. The default json library is Jason. To use a different library, set the :json_library config to the module of your choice, i.e: config :arangox, :json_library, Poison Examples iex&gt; {:ok, conn} = Arangox.start_link(client: Arangox.GunClient) iex&gt; Arangox.request(conn, :options, &quot;/&quot;) {:ok, %Arangox.Request{ body: &quot;&quot;, headers: %{&quot;authorization&quot; =&gt; &quot;...&quot;}, method: :options, path: &quot;/&quot; }, %Arangox.Response{ body: nil, headers: %{ &quot;allow&quot; =&gt; &quot;DELETE, GET, HEAD, OPTIONS, PATCH, POST, PUT&quot;, &quot;connection&quot; =&gt; &quot;Keep-Alive&quot;, &quot;content-length&quot; =&gt; &quot;0&quot;, &quot;content-type&quot; =&gt; &quot;text/plain; charset=utf-8&quot;, &quot;server&quot; =&gt; &quot;ArangoDB&quot;, &quot;x-content-type-options&quot; =&gt; &quot;nosniff&quot; }, status: 200 }} Start Options Arangox assumes defaults for the :endpoints, :username and :password options, and db_connection assumes a default :pool_size of 1, so the following: Arangox.start_link() Is equivalent to: options = [ endpoints: &quot;http://localhost:8529&quot;, username: &quot;root&quot;, password: &quot;&quot;, pool_size: 1 ] Arangox.start_link(options) Endpoints Unencrypted endpoints can be specified with either http:// or tcp://, whereas encrypted endpoints can be specified with https://, ssl:// or tls://: &quot;tcp://localhost:8529&quot; == &quot;http://localhost:8529&quot; &quot;https://localhost:8529&quot; == &quot;ssl://localhost:8529&quot; == &quot;tls://localhost:8529&quot; &quot;tcp+unix:///tmp/arangodb.sock&quot; == &quot;http+unix:///tmp/arangodb.sock&quot; &quot;https+unix:///tmp/arangodb.sock&quot; == &quot;ssl+unix:///tmp/arangodb.sock&quot; == &quot;tls+unix:///tmp/arangodb.sock&quot; &quot;tcp://unix:/tmp/arangodb.sock&quot; == &quot;http://unix:/tmp/arangodb.sock&quot; &quot;https://unix:/tmp/arangodb.sock&quot; == &quot;ssl://unix:/tmp/arangodb.sock&quot; == &quot;tls://unix:/tmp/arangodb.sock&quot; The :endpoints option accepts either a binary, or a list of binaries. In the case of a list, Arangox will try to establish a connection with the first endpoint it can. If a connection is established, the availability of the server will be checked (via the ArangoDB api), and if an endpoint is in maintenance mode or is a Follower in an Active Failover setup, the connection will be dropped, or in the case of a list, the endpoint skipped. With the :read_only? option set to true, arangox will try to find a server in readonly mode instead and add the x-arango-allow-dirty-read header to every request: iex&gt; endpoints = [&quot;http://localhost:8003&quot;, &quot;http://localhost:8004&quot;, &quot;http://localhost:8005&quot;] iex&gt; {:ok, conn} = Arangox.start_link(endpoints: endpoints, read_only?: true) iex&gt; %Arangox.Response{body: body} = Arangox.get!(conn, &quot;/_admin/server/mode&quot;) iex&gt; body[&quot;mode&quot;] &quot;readonly&quot; iex&gt; {:error, exception} = Arangox.post(conn, &quot;/_api/database&quot;, %{name: &quot;newDatabase&quot;}) iex&gt; exception.message &quot;forbidden&quot; Authentication Velocy When using the default client, authorization is resolved with the :username and :password options after a connection is established (authorization headers are not used). This can be disabled by setting the :auth? option to false. HTTP When using an HTTP client, Arangox will generate a Basic authorization header with the :username and :password options and add it to every request. To prevent this behavior, set the :auth? option to false. iex&gt; {:ok, conn} = Arangox.start_link(auth?: false, client: Arangox.GunClient) iex&gt; {:error, exception} = Arangox.get(conn, &quot;/_admin/server/mode&quot;) iex&gt; exception.message &quot;not authorized to execute this request&quot; The header value is obfuscated in transfomed requests returned by arangox, for obvious reasons: iex&gt; {:ok, conn} = Arangox.start_link(client: Arangox.GunClient) iex&gt; {:ok, request, _response} = Arangox.options(conn, &quot;/&quot;) iex&gt; request.headers %{&quot;authorization&quot; =&gt; &quot;...&quot;} Databases Velocy If the :database option is set, it can be overridden by prepending the path of a request with /_db/:value. If nothing is set, the request will be sent as-is and ArangoDB will assume the _system database. HTTP When using an HTTP client, arangox will prepend /_db/:value to the path of every request only if it isn&#39;t already prepended. If the start option is not set, nothing is prepended. iex&gt; {:ok, conn} = Arangox.start_link(client: Arangox.GunClient) iex&gt; {:ok, request, _response} = Arangox.get(conn, &quot;/_admin/time&quot;) iex&gt; request.path &quot;/_admin/time&quot; iex&gt; {:ok, conn} = Arangox.start_link(database: &quot;_system&quot;, client: Arangox.GunClient) iex&gt; {:ok, request, _response} = Arangox.get(conn, &quot;/_admin/time&quot;) iex&gt; request.path &quot;/_db/_system/_admin/time&quot; iex&gt; {:ok, request, _response} = Arangox.get(conn, &quot;/_db/_system/_admin/time&quot;) iex&gt; request.path &quot;/_db/_system/_admin/time&quot; Headers Headers given to the start option are merged with every request, but will not override any of the headers set by Arangox: iex&gt; {:ok, conn} = Arangox.start_link(headers: %{&quot;header&quot; =&gt; &quot;value&quot;}) iex&gt; {:ok, request, _response} = Arangox.get(conn, &quot;/_api/version&quot;) iex&gt; request.headers %{&quot;header&quot; =&gt; &quot;value&quot;} Headers passed to requests will override any of the headers given to the start option or set by Arangox: iex&gt; {:ok, conn} = Arangox.start_link(headers: %{&quot;header&quot; =&gt; &quot;value&quot;}) iex&gt; {:ok, request, _response} = Arangox.get(conn, &quot;/_api/version&quot;, %{&quot;header&quot; =&gt; &quot;new_value&quot;}) iex&gt; request.headers %{&quot;header&quot; =&gt; &quot;new_value&quot;} Transport The :connect_timeout start option defaults to 5_000. Transport options can be specified via :tcp_opts and :ssl_opts, for unencrypted and encrypted connections respectively. When using :gun or :mint, these options are passed directly to the :transport_opts connect option. See :gen_tcp.connect_option() for more information on :tcp_opts, or :ssl.tls_client_option() for :ssl_opts. The :client_opts option can be used to pass client-specific options to :gun or :mint. These options are merged with and may override values set by arangox. Some options cannot be overridden (i.e. :mint&#39;s :mode option). If :transport_opts is set here it will override everything given to :tcp_opts or :ssl_opts, regardless of whether or not a connection is encrypted. See the gun:opts() type in the gun docs or connect/4 in the mint docs for more information. Request Options Request options are handled by and passed directly to :db_connection. See execute/4 in the :db_connection docs for supported options. Request timeouts default to 15_000. iex&gt; {:ok, conn} = Arangox.start_link() iex&gt; Arangox.get!(conn, &quot;/_admin/server/availability&quot;, [], timeout: 15_000) %Arangox.Response{ body: %{&quot;code&quot; =&gt; 200, &quot;error&quot; =&gt; false, &quot;mode&quot; =&gt; &quot;default&quot;}, headers: %{}, status: 200 } Contributing mix do format, credo --strict docker-compose up -d mix test Roadmap An Ecto adapter More descriptive logs"},{"ref":"Arangox.html#abort/2","title":"Arangox.abort/2","type":"function","doc":"Aborts a transaction for the given reason. Delegates to DBConnection.abort/2. Example iex&gt; {:ok, conn} = Arangox.start_link() iex&gt; Arangox.transaction(conn, fn c -&gt; iex&gt; Arangox.abort(c, :reason) iex&gt; end) {:error, :reason}"},{"ref":"Arangox.html#child_spec/1","title":"Arangox.child_spec/1","type":"function","doc":"Returns a supervisor child specification for a DBConnection pool."},{"ref":"Arangox.html#cursor/4","title":"Arangox.cursor/4","type":"function","doc":"Creates a cursor and returns a DBConnection.Stream struct. Results are fetched upon enumeration. The cursor is created, results fetched, then deleted from the database upon each enumeration (not to be confused with iteration). When a cursor is created, an initial result set is fetched from the database. The initial result is returned with the first iteration, subsequent iterations are fetched lazily. Can only be used within a transaction/3 or run/3 call. Accepts any of the options accepted by DBConnection.stream/4, as well as any of the following: :properties - A list or map of additional body attributes to append to the request body when creating the cursor. Delegates to DBConnection.stream/4. Example iex&gt; {:ok, conn} = Arangox.start_link() iex&gt; Arangox.transaction(conn, fn c -&gt; iex&gt; stream = iex&gt; Arangox.cursor( iex&gt; c, iex&gt; &quot;FOR i IN [1, 2, 3] FILTER i == 1 || i == @num RETURN i&quot;, iex&gt; %{num: 2}, iex&gt; properties: [batchSize: 1] iex&gt; ) iex&gt; iex&gt; first_batch = Enum.at(stream, 0).body[&quot;result&quot;] iex&gt; iex&gt; exhaust_cursor = iex&gt; Enum.reduce(stream, [], fn resp, acc -&gt; iex&gt; acc ++ resp.body[&quot;result&quot;] iex&gt; end) iex&gt; iex&gt; {first_batch, exhaust_cursor} iex&gt; end) {:ok, {[1], [1, 2]}}"},{"ref":"Arangox.html#delete/4","title":"Arangox.delete/4","type":"function","doc":"Runs a DELETE request against a connection pool. Accepts any of the options accepted by DBConnection.execute/4."},{"ref":"Arangox.html#delete!/4","title":"Arangox.delete!/4","type":"function","doc":"Runs a DELETE request against a connection pool. Raises in the case of an error. Accepts any of the options accepted by DBConnection.execute!/4."},{"ref":"Arangox.html#get/4","title":"Arangox.get/4","type":"function","doc":"Runs a GET request against a connection pool. Accepts any of the options accepted by DBConnection.execute/4."},{"ref":"Arangox.html#get!/4","title":"Arangox.get!/4","type":"function","doc":"Runs a GET request against a connection pool. Raises in the case of an error. Accepts any of the options accepted by DBConnection.execute!/4."},{"ref":"Arangox.html#head/4","title":"Arangox.head/4","type":"function","doc":"Runs a HEAD request against a connection pool. Accepts any of the options accepted by DBConnection.execute/4."},{"ref":"Arangox.html#head!/4","title":"Arangox.head!/4","type":"function","doc":"Runs a HEAD request against a connection pool. Raises in the case of an error. Accepts any of the options accepted by DBConnection.execute!/4."},{"ref":"Arangox.html#json_library/0","title":"Arangox.json_library/0","type":"function","doc":"Returns the configured JSON library. To change the library, include the following in your config/config.exs: config :arangox, :json_library, Module Defaults to Jason."},{"ref":"Arangox.html#options/4","title":"Arangox.options/4","type":"function","doc":"Runs a OPTIONS request against a connection pool. Accepts any of the options accepted by DBConnection.execute/4."},{"ref":"Arangox.html#options!/4","title":"Arangox.options!/4","type":"function","doc":"Runs a OPTIONS request against a connection pool. Raises in the case of an error. Accepts any of the options accepted by DBConnection.execute!/4."},{"ref":"Arangox.html#patch/5","title":"Arangox.patch/5","type":"function","doc":"Runs a PATCH request against a connection pool. Accepts any of the options accepted by DBConnection.execute/4."},{"ref":"Arangox.html#patch!/5","title":"Arangox.patch!/5","type":"function","doc":"Runs a PATCH request against a connection pool. Raises in the case of an error. Accepts any of the options accepted by DBConnection.execute!/4."},{"ref":"Arangox.html#post/5","title":"Arangox.post/5","type":"function","doc":"Runs a POST request against a connection pool. Accepts any of the options accepted by DBConnection.execute/4."},{"ref":"Arangox.html#post!/5","title":"Arangox.post!/5","type":"function","doc":"Runs a POST request against a connection pool. Raises in the case of an error. Accepts any of the options accepted by DBConnection.execute!/4."},{"ref":"Arangox.html#put/5","title":"Arangox.put/5","type":"function","doc":"Runs a PUT request against a connection pool. Accepts any of the options accepted by DBConnection.execute/4."},{"ref":"Arangox.html#put!/5","title":"Arangox.put!/5","type":"function","doc":"Runs a PUT request against a connection pool. Raises in the case of an error. Accepts any of the options accepted by DBConnection.execute!/4."},{"ref":"Arangox.html#request/6","title":"Arangox.request/6","type":"function","doc":"Runs a request against a connection pool. Accepts any of the options accepted by DBConnection.execute/4."},{"ref":"Arangox.html#request!/6","title":"Arangox.request!/6","type":"function","doc":"Runs a request against a connection pool. Raises in the case of an error. Accepts any of the options accepted by DBConnection.execute!/4."},{"ref":"Arangox.html#run/3","title":"Arangox.run/3","type":"function","doc":"Acquires a connection from a pool and runs a series of requests or cursors with it. If the connection disconnects, all future calls using that connection reference will fail. Runs can be nested multiple times if the connection reference is used to start a nested run (i.e. calling another function that calls this one). The top level run function will represent the actual run. Delegates to DBConnection.run/3. Example result = Arangox.run(conn, fn c -&gt; Arangox.request!(c, ...) end)"},{"ref":"Arangox.html#start_link/1","title":"Arangox.start_link/1","type":"function","doc":"Starts a connection pool. Options Accepts any of the options accepted by DBConnection.start_link/2, as well as any of the following: :endpoints - Either a single ArangoDB endpoint binary, or a list of endpoints in order of presedence. Each process in a pool will individually attempt to establish a connection with and check the availablility of each endpoint in the order given until an available endpoint is found. Defaults to &quot;http://localhost:8529&quot;. :database - Arangox will prepend /_db/:value to the path of every request that isn&#39;t already prepended. If a value is not given, nothing is prepended (ArangoDB will assume the _system database). :headers - A map of headers to merge with every request. :disconnect_on_error_codes - A list of status codes that will trigger a forced disconnect. Only integers within the range 400..599 are affected. Defaults to [401, 405, 503, 505]. :auth? - Configure whether or not to resolve authorization (with the :username and :password options). Defaults to true. :username - Defaults to &quot;root&quot;. :password - Defaults to &quot;&quot;. :read_only? - Read-only pools will only connect to followers in an active failover setup and add an x-arango-allow-dirty-read header to every request. Defaults to false. :connect_timeout - Sets the timeout for establishing connections with a database. :tcp_opts - Transport options for the tcp socket interface (:gen_tcp in the case of gun or mint). :ssl_opts - Transport options for the ssl socket interface (:ssl in the case of gun or mint). :client - A module that implements the Arangox.Client behaviour. Defaults to Arangox.VelocyClient. :client_opts - Options for the client library being used. WARNING: If :transport_opts is set here it will override the options given to :tcp_opts and :ssl_opts. :failover_callback - A function to call every time arangox fails to establish a connection. This is only called if a list of endpoints is given, regardless of whether or not it&#39;s connecting to an endpoint in an active failover setup. Can be either an anonymous function that takes one argument (which is an %Arangox.Error{} struct), or a three-element tuple containing arguments to pass to apply/3 (in which case an %Arangox.Error{} struct is always prepended to the arguments)."},{"ref":"Arangox.html#status/1","title":"Arangox.status/1","type":"function","doc":"Fetches the current status of a transaction from the database and returns its corresponding DBconnection status. Delegates to DBConnection.status/1."},{"ref":"Arangox.html#transaction/3","title":"Arangox.transaction/3","type":"function","doc":"Acquires a connection from a pool, begins a transaction in the database and runs a series of requests or cursors with it. If the connection disconnects, all future calls using that connection reference will fail. Transactions can be nested multiple times if the connection reference is used to start a nested transactions (i.e. calling another function that calls this one). The top level transaction function will represent the actual transaction and nested transactions will be interpreted as a run/3, erego, any collections declared in nested transactions will have no effect. Accepts any of the options accepted by DBConnection.transaction/3, as well as any of the following: :read - An array of collection names or a single collection name as a binary. :write - An array of collection names or a single collection name as a binary. :exclusive - An array of collection names or a single collection name as a binary. :properties - A list or map of additional body attributes to append to the request body when beginning a transaction. Delegates to DBConnection.transaction/3. Example Arangox.transaction(conn, fn c -&gt; Arangox.status(c) #=&gt; :transaction # do stuff end, [ write: &quot;something&quot;, properties: [waitForSync: true] ])"},{"ref":"Arangox.html#t:bindvars/0","title":"Arangox.bindvars/0","type":"type","doc":""},{"ref":"Arangox.html#t:body/0","title":"Arangox.body/0","type":"type","doc":""},{"ref":"Arangox.html#t:client/0","title":"Arangox.client/0","type":"type","doc":""},{"ref":"Arangox.html#t:conn/0","title":"Arangox.conn/0","type":"type","doc":""},{"ref":"Arangox.html#t:endpoint/0","title":"Arangox.endpoint/0","type":"type","doc":""},{"ref":"Arangox.html#t:headers/0","title":"Arangox.headers/0","type":"type","doc":""},{"ref":"Arangox.html#t:method/0","title":"Arangox.method/0","type":"type","doc":""},{"ref":"Arangox.html#t:path/0","title":"Arangox.path/0","type":"type","doc":""},{"ref":"Arangox.html#t:query/0","title":"Arangox.query/0","type":"type","doc":""},{"ref":"Arangox.html#t:start_option/0","title":"Arangox.start_option/0","type":"type","doc":""},{"ref":"Arangox.Client.html","title":"Arangox.Client","type":"behaviour","doc":"HTTP client behaviour for Arangox. Arangox uses client implementations to perform all it&#39;s connection and execution operations. To use an http library other than :gun or :mint, implement this behaviour in a module and pass that module to the :client start option."},{"ref":"Arangox.Client.html#alive?/1","title":"Arangox.Client.alive?/1","type":"function","doc":""},{"ref":"Arangox.Client.html#c:alive?/1","title":"Arangox.Client.alive?/1","type":"callback","doc":""},{"ref":"Arangox.Client.html#close/1","title":"Arangox.Client.close/1","type":"function","doc":""},{"ref":"Arangox.Client.html#c:close/1","title":"Arangox.Client.close/1","type":"callback","doc":""},{"ref":"Arangox.Client.html#c:connect/2","title":"Arangox.Client.connect/2","type":"callback","doc":"Receives an Arangox.Endpoint struct and all the start options from Arangox.start_link/1. The socket returned from this callback gets placed in the :socket field of an Arango.Connection struct (a connection&#39;s state) to be used by the other callbacks as needed. It can be anything, a tuple, another struct, whatever the client needs. It&#39;s up to the client to consolidate the :connect_timeout, :transport_opts and :client_opts options."},{"ref":"Arangox.Client.html#connect/3","title":"Arangox.Client.connect/3","type":"function","doc":""},{"ref":"Arangox.Client.html#request/2","title":"Arangox.Client.request/2","type":"function","doc":""},{"ref":"Arangox.Client.html#c:request/2","title":"Arangox.Client.request/2","type":"callback","doc":"Receives a Arangox.Request struct and a connection&#39;s state (an Arangox.Connection struct), and returns an Arangox.Response struct or error (or exception struct), along with the new state (which doesn&#39;t necessarily need to change). Arangox handles the encoding and decoding of request and response bodies, and merging headers. If a connection is lost, this may return {:error, :noproc, state} to force a disconnect, otherwise an attempt to reconnect may not be made until the next request hitting this process fails."},{"ref":"Arangox.Client.html#t:exception_or_reason/0","title":"Arangox.Client.exception_or_reason/0","type":"type","doc":""},{"ref":"Arangox.Client.html#t:socket/0","title":"Arangox.Client.socket/0","type":"type","doc":""},{"ref":"Arangox.Endpoint.html","title":"Arangox.Endpoint","type":"module","doc":"Utilities for parsing ArangoDB endpoints. iex&gt; Endpoint.new(&quot;http://localhost:8529&quot;) %Arangox.Endpoint{addr: {:tcp, &quot;localhost&quot;, 8529}, ssl?: false} iex&gt; Endpoint.new(&quot;https://localhost:8529&quot;) %Arangox.Endpoint{addr: {:tcp, &quot;localhost&quot;, 8529}, ssl?: true} iex&gt; Endpoint.new(&quot;http://unix:/tmp/arangodb.sock&quot;) %Arangox.Endpoint{addr: {:unix, &quot;/tmp/arangodb.sock&quot;}, ssl?: false}"},{"ref":"Arangox.Endpoint.html#new/1","title":"Arangox.Endpoint.new/1","type":"function","doc":"Parses an endpoint and returns an %Arangox.Endpoint{} struct."},{"ref":"Arangox.Endpoint.html#t:addr/0","title":"Arangox.Endpoint.addr/0","type":"type","doc":""},{"ref":"Arangox.Endpoint.html#t:t/0","title":"Arangox.Endpoint.t/0","type":"type","doc":""},{"ref":"Arangox.Error.html","title":"Arangox.Error","type":"exception","doc":""},{"ref":"Arangox.Error.html#t:t/0","title":"Arangox.Error.t/0","type":"type","doc":""},{"ref":"Arangox.GunClient.html","title":"Arangox.GunClient","type":"module","doc":"An HTTP client implementation of the :gun library. Requires :gun to be added as a dependency. Hex.pm Documentation"},{"ref":"Arangox.MintClient.html","title":"Arangox.MintClient","type":"module","doc":"An HTTP client implementation of the :mint library. Requires :mint to be added as a dependency. Hex.pm Documentation"},{"ref":"Arangox.Request.html","title":"Arangox.Request","type":"module","doc":""},{"ref":"Arangox.Request.html#t:t/0","title":"Arangox.Request.t/0","type":"type","doc":""},{"ref":"Arangox.Response.html","title":"Arangox.Response","type":"module","doc":""},{"ref":"Arangox.Response.html#t:t/0","title":"Arangox.Response.t/0","type":"type","doc":""},{"ref":"Arangox.VelocyClient.html","title":"Arangox.VelocyClient","type":"module","doc":"The default client. Implements the VelocyStream protocol. URI query parsing functions proudly stolen from Plataformatec and licensed under Apache 2.0."},{"ref":"Arangox.VelocyClient.html#authorize/1","title":"Arangox.VelocyClient.authorize/1","type":"function","doc":""},{"ref":"Arangox.VelocyClient.html#vst_maxsize/0","title":"Arangox.VelocyClient.vst_maxsize/0","type":"function","doc":"Returns the configured maximum size (in bytes) for a VelocyPack chunk. To change the chunk size, include the following in your config/config.exs: config :arangox, :vst_maxsize, 12_345 Defaults to 30_720."},{"ref":"readme.html","title":"Arangox","type":"extras","doc":"Arangox An implementation of :db_connection for ArangoDB. Supports VelocyStream, Active Failover, transactions and cursors. Documentation Tested on: ArangoDB 3.3.9 - 3.5 Elixir 1.6 - 1.9 OTP 20 - 22"},{"ref":"readme.html#examples","title":"Arangox - Examples","type":"extras","doc":"iex&gt; {:ok, conn} = Arangox.start_link(pool_size: 10) iex&gt; Arangox.request(conn, :get, &quot;/_admin/server/availability&quot;) {:ok, %Arangox.Request{ body: &quot;&quot;, headers: %{}, method: :get, path: &quot;/_admin/server/availability&quot; }, %Arangox.Response{ body: %{&quot;code&quot; =&gt; 200, &quot;error&quot; =&gt; false, &quot;mode&quot; =&gt; &quot;default&quot;}, headers: %{}, status: 200 }} iex&gt; Arangox.get!(conn, &quot;/_admin/server/availability&quot;) %Arangox.Response{ body: %{&quot;code&quot; =&gt; 200, &quot;error&quot; =&gt; false, &quot;mode&quot; =&gt; &quot;default&quot;}, headers: %{}, status: 200 } iex&gt; Arangox.get(conn, &quot;/invalid&quot;) {:error, %Arangox.Error{ endpoint: &quot;http://localhost:8529&quot;, error_num: 404, message: &quot;unknown path &#39;/invalid&#39;&quot;, status: 404 }} iex&gt; Arangox.transaction(conn, fn c -&gt; iex&gt; stream = iex&gt; Arangox.cursor( iex&gt; c, iex&gt; &quot;FOR i IN [1, 2, 3] FILTER i == 1 || i == @num RETURN i&quot;, iex&gt; %{num: 2}, iex&gt; properties: [batchSize: 1] iex&gt; ) iex&gt; iex&gt; Enum.reduce(stream, [], fn resp, acc -&gt; iex&gt; acc ++ resp.body[&quot;result&quot;] iex&gt; end) iex&gt; end) {:ok, [1, 2]}"},{"ref":"readme.html#peer-dependencies","title":"Arangox - Peer Dependencies","type":"extras","doc":"By default, Arangox communicates with ArangoDB via the VelocyStream protocol, which requires the :velocy library: def deps do [ ... {:arangox, &quot;~&gt; 0.1.0&quot;}, {:velocy, &quot;~&gt; 1.1&quot;} ] end The default vst chunk size is 30_720. To change it, you can include the following in your config/config.exs: config :arangox, :vst_maxsize, 12_345 HTTP Arangox has two HTTP clients, Arangox.GunClient and Arangox.MintClient, they require a json library: def deps do [ ... {:arangox, &quot;~&gt; 0.1.0&quot;}, {:jason, &quot;~&gt; 1.1&quot;}, {:gun, &quot;~&gt; 1.3&quot;} # or {:mint, &quot;~&gt; 0.4.0&quot;} ] end Arangox.start_link(client: Arangox.GunClient) # or Arangox.MintClient NOTE: :mint doesn&#39;t support unix sockets. NOTE: Since :gun is an Erlang library, you might need to add it as an extra application in mix.exs: def application() do [ extra_applications: [:logger, :gun]) ] end To use something else, you&#39;d have to implement the Arangox.Client behaviour in a module somewhere and set that instead. The default json library is Jason. To use a different library, set the :json_library config to the module of your choice, i.e: config :arangox, :json_library, Poison Examples iex&gt; {:ok, conn} = Arangox.start_link(client: Arangox.GunClient) iex&gt; Arangox.request(conn, :options, &quot;/&quot;) {:ok, %Arangox.Request{ body: &quot;&quot;, headers: %{&quot;authorization&quot; =&gt; &quot;...&quot;}, method: :options, path: &quot;/&quot; }, %Arangox.Response{ body: nil, headers: %{ &quot;allow&quot; =&gt; &quot;DELETE, GET, HEAD, OPTIONS, PATCH, POST, PUT&quot;, &quot;connection&quot; =&gt; &quot;Keep-Alive&quot;, &quot;content-length&quot; =&gt; &quot;0&quot;, &quot;content-type&quot; =&gt; &quot;text/plain; charset=utf-8&quot;, &quot;server&quot; =&gt; &quot;ArangoDB&quot;, &quot;x-content-type-options&quot; =&gt; &quot;nosniff&quot; }, status: 200 }}"},{"ref":"readme.html#start-options","title":"Arangox - Start Options","type":"extras","doc":"Arangox assumes defaults for the :endpoints, :username and :password options, and db_connection assumes a default :pool_size of 1, so the following: Arangox.start_link() Is equivalent to: options = [ endpoints: &quot;http://localhost:8529&quot;, username: &quot;root&quot;, password: &quot;&quot;, pool_size: 1 ] Arangox.start_link(options)"},{"ref":"readme.html#endpoints","title":"Arangox - Endpoints","type":"extras","doc":"Unencrypted endpoints can be specified with either http:// or tcp://, whereas encrypted endpoints can be specified with https://, ssl:// or tls://: &quot;tcp://localhost:8529&quot; == &quot;http://localhost:8529&quot; &quot;https://localhost:8529&quot; == &quot;ssl://localhost:8529&quot; == &quot;tls://localhost:8529&quot; &quot;tcp+unix:///tmp/arangodb.sock&quot; == &quot;http+unix:///tmp/arangodb.sock&quot; &quot;https+unix:///tmp/arangodb.sock&quot; == &quot;ssl+unix:///tmp/arangodb.sock&quot; == &quot;tls+unix:///tmp/arangodb.sock&quot; &quot;tcp://unix:/tmp/arangodb.sock&quot; == &quot;http://unix:/tmp/arangodb.sock&quot; &quot;https://unix:/tmp/arangodb.sock&quot; == &quot;ssl://unix:/tmp/arangodb.sock&quot; == &quot;tls://unix:/tmp/arangodb.sock&quot; The :endpoints option accepts either a binary, or a list of binaries. In the case of a list, Arangox will try to establish a connection with the first endpoint it can. If a connection is established, the availability of the server will be checked (via the ArangoDB api), and if an endpoint is in maintenance mode or is a Follower in an Active Failover setup, the connection will be dropped, or in the case of a list, the endpoint skipped. With the :read_only? option set to true, arangox will try to find a server in readonly mode instead and add the x-arango-allow-dirty-read header to every request: iex&gt; endpoints = [&quot;http://localhost:8003&quot;, &quot;http://localhost:8004&quot;, &quot;http://localhost:8005&quot;] iex&gt; {:ok, conn} = Arangox.start_link(endpoints: endpoints, read_only?: true) iex&gt; %Arangox.Response{body: body} = Arangox.get!(conn, &quot;/_admin/server/mode&quot;) iex&gt; body[&quot;mode&quot;] &quot;readonly&quot; iex&gt; {:error, exception} = Arangox.post(conn, &quot;/_api/database&quot;, %{name: &quot;newDatabase&quot;}) iex&gt; exception.message &quot;forbidden&quot;"},{"ref":"readme.html#authentication","title":"Arangox - Authentication","type":"extras","doc":"Velocy When using the default client, authorization is resolved with the :username and :password options after a connection is established (authorization headers are not used). This can be disabled by setting the :auth? option to false. HTTP When using an HTTP client, Arangox will generate a Basic authorization header with the :username and :password options and add it to every request. To prevent this behavior, set the :auth? option to false. iex&gt; {:ok, conn} = Arangox.start_link(auth?: false, client: Arangox.GunClient) iex&gt; {:error, exception} = Arangox.get(conn, &quot;/_admin/server/mode&quot;) iex&gt; exception.message &quot;not authorized to execute this request&quot; The header value is obfuscated in transfomed requests returned by arangox, for obvious reasons: iex&gt; {:ok, conn} = Arangox.start_link(client: Arangox.GunClient) iex&gt; {:ok, request, _response} = Arangox.options(conn, &quot;/&quot;) iex&gt; request.headers %{&quot;authorization&quot; =&gt; &quot;...&quot;}"},{"ref":"readme.html#databases","title":"Arangox - Databases","type":"extras","doc":"Velocy If the :database option is set, it can be overridden by prepending the path of a request with /_db/:value. If nothing is set, the request will be sent as-is and ArangoDB will assume the _system database. HTTP When using an HTTP client, arangox will prepend /_db/:value to the path of every request only if it isn&#39;t already prepended. If the start option is not set, nothing is prepended. iex&gt; {:ok, conn} = Arangox.start_link(client: Arangox.GunClient) iex&gt; {:ok, request, _response} = Arangox.get(conn, &quot;/_admin/time&quot;) iex&gt; request.path &quot;/_admin/time&quot; iex&gt; {:ok, conn} = Arangox.start_link(database: &quot;_system&quot;, client: Arangox.GunClient) iex&gt; {:ok, request, _response} = Arangox.get(conn, &quot;/_admin/time&quot;) iex&gt; request.path &quot;/_db/_system/_admin/time&quot; iex&gt; {:ok, request, _response} = Arangox.get(conn, &quot;/_db/_system/_admin/time&quot;) iex&gt; request.path &quot;/_db/_system/_admin/time&quot;"},{"ref":"readme.html#headers","title":"Arangox - Headers","type":"extras","doc":"Headers given to the start option are merged with every request, but will not override any of the headers set by Arangox: iex&gt; {:ok, conn} = Arangox.start_link(headers: %{&quot;header&quot; =&gt; &quot;value&quot;}) iex&gt; {:ok, request, _response} = Arangox.get(conn, &quot;/_api/version&quot;) iex&gt; request.headers %{&quot;header&quot; =&gt; &quot;value&quot;} Headers passed to requests will override any of the headers given to the start option or set by Arangox: iex&gt; {:ok, conn} = Arangox.start_link(headers: %{&quot;header&quot; =&gt; &quot;value&quot;}) iex&gt; {:ok, request, _response} = Arangox.get(conn, &quot;/_api/version&quot;, %{&quot;header&quot; =&gt; &quot;new_value&quot;}) iex&gt; request.headers %{&quot;header&quot; =&gt; &quot;new_value&quot;}"},{"ref":"readme.html#transport","title":"Arangox - Transport","type":"extras","doc":"The :connect_timeout start option defaults to 5_000. Transport options can be specified via :tcp_opts and :ssl_opts, for unencrypted and encrypted connections respectively. When using :gun or :mint, these options are passed directly to the :transport_opts connect option. See :gen_tcp.connect_option() for more information on :tcp_opts, or :ssl.tls_client_option() for :ssl_opts. The :client_opts option can be used to pass client-specific options to :gun or :mint. These options are merged with and may override values set by arangox. Some options cannot be overridden (i.e. :mint&#39;s :mode option). If :transport_opts is set here it will override everything given to :tcp_opts or :ssl_opts, regardless of whether or not a connection is encrypted. See the gun:opts() type in the gun docs or connect/4 in the mint docs for more information."},{"ref":"readme.html#request-options","title":"Arangox - Request Options","type":"extras","doc":"Request options are handled by and passed directly to :db_connection. See execute/4 in the :db_connection docs for supported options. Request timeouts default to 15_000. iex&gt; {:ok, conn} = Arangox.start_link() iex&gt; Arangox.get!(conn, &quot;/_admin/server/availability&quot;, [], timeout: 15_000) %Arangox.Response{ body: %{&quot;code&quot; =&gt; 200, &quot;error&quot; =&gt; false, &quot;mode&quot; =&gt; &quot;default&quot;}, headers: %{}, status: 200 }"},{"ref":"readme.html#contributing","title":"Arangox - Contributing","type":"extras","doc":"mix do format, credo --strict docker-compose up -d mix test"},{"ref":"readme.html#roadmap","title":"Arangox - Roadmap","type":"extras","doc":"An Ecto adapter More descriptive logs"}]